package java.rmi.server;

import java.rmi.NoSuchObjectException;
import sun.rmi.transport.ObjectTable;
import sun.rmi.server.UnicastServerRef2;
import sun.rmi.server.UnicastServerRef;
import java.rmi.Remote;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.rmi.RemoteException;

public class UnicastRemoteObject extends RemoteServer
{
    private int port;
    private RMIClientSocketFactory csf;
    private RMIServerSocketFactory ssf;
    private static final long serialVersionUID = 4974527148936298033L;
    
    protected UnicastRemoteObject() throws RemoteException {
        this(0);
    }
    
    protected UnicastRemoteObject(int port) throws RemoteException {
        this.port = 0;
        this.csf = null;
        this.ssf = null;
        exportObject(this, this.port = port);
    }
    
    protected UnicastRemoteObject(int port, RMIClientSocketFactory csf, RMIServerSocketFactory ssf) throws RemoteException {
        this.port = 0;
        this.csf = null;
        this.ssf = null;
        exportObject(this, this.port = port, this.csf = csf, this.ssf = ssf);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.reexport();
    }
    
    public Object clone() throws CloneNotSupportedException {
        try {
            UnicastRemoteObject unicastRemoteObject = (UnicastRemoteObject)super.clone();
            unicastRemoteObject.reexport();
            return unicastRemoteObject;
        }
        catch (RemoteException ex) {
            throw new ServerCloneException("Clone failed", ex);
        }
    }
    
    private void reexport() throws RemoteException {
        if (this.csf == null && this.ssf == null) {
            exportObject(this, this.port);
        }
        else {
            exportObject(this, this.port, this.csf, this.ssf);
        }
    }
    
    @Deprecated
    public static RemoteStub exportObject(Remote remote) throws RemoteException {
        return (RemoteStub)exportObject(remote, new UnicastServerRef(true));
    }
    
    public static Remote exportObject(Remote remote, int port) throws RemoteException {
        return exportObject(remote, new UnicastServerRef(port));
    }
    
    public static Remote exportObject(Remote remote, int port, RMIClientSocketFactory csf, RMIServerSocketFactory ssf) throws RemoteException {
        return exportObject(remote, new UnicastServerRef2(port, csf, ssf));
    }
    
    public static boolean unexportObject(Remote obj, boolean force) throws NoSuchObjectException {
        return ObjectTable.unexportObject(obj, force);
    }
    
    private static Remote exportObject(Remote impl, UnicastServerRef ref) throws RemoteException {
        if ((impl instanceof UnicastRemoteObject)) {
            ((UnicastRemoteObject)impl).ref = ref;
        }
        return ref.exportObject(impl, null, false);
    }
}
